###Andrea Ravignani, August 2017
###Supplementary material to:
###Ravignani, A. (in review). Spontaneous rhythms in a harbor seal pup calls. BMC Research Notes.

###This script requires that a wave file (.wav) and its corresponding Praat Textgrid file (.TextGrid)
###have the same name and are both in the same directory as this script

import parselmouth
import numpy as np
import tgt
import os
import math
from scipy.stats import ks_2samp

##adjustable sound parameters to calculate intesity
timestep=0.001
minimumpitch=100


###given a Praat intensity object and an interval from a tier
###it outputs intensity and duration information
def combine_wav_textgrid(interval,intensity,asealname,date,time):
    end=int(float(interval[1])/timestep)
    start=int(float(interval[0])/timestep)
    onset=int(float(interval[0])*1000)
    duration=int((float(interval[1])-float(interval[0]))*1000)
    annotated_intensity=list(intensity.values[start:end])
    max_intensity = max(annotated_intensity)
    min_intensity = min(annotated_intensity)
    delta_intensity = float(max_intensity/min_intensity)
    time_peak_intensity = 100*annotated_intensity.index(max_intensity)/len(annotated_intensity)
    line=[str(asealname),date,time,onset,duration,delta_intensity,time_peak_intensity]
    return line

def find_time_max_intensity(intensity,start,end):
    annotated_intensity=list(intensity.values[start:end])
    max_intensity = max(annotated_intensity)
    time_peak_intensity = start+(1000*annotated_intensity.index(max_intensity)/len(annotated_intensity))
    return time_peak_intensity


def get_data_from_tier(tg,intensity,asealname,start_timepoint= 60,end_timepoint =540):
    tiers=[]
    durations=[]
    time_peak_intensities=[]
    #find info specific seal
    aseal=tg.get_tier_by_name(asealname)
    #calculate nmb call per minute
    numb_calls_given_time = aseal.get_annotations_between_timepoints(start_timepoint,end_timepoint)
    calls_per_minute = len(numb_calls_given_time)*60/float(end_timepoint-start_timepoint)
    print calls_per_minute
    #extract good pup MACs (denoted by u) and put them in list
    cleanseal=aseal.get_annotations_with_matching_text(r'u')
    for interval_nmb in range(len(cleanseal)):
        start=int(1000*cleanseal[interval_nmb].start_time)
        end=int(1000*cleanseal[interval_nmb].end_time)
        time_peak_intensity=find_time_max_intensity(intensity,start,end)
        duration=end-start
        tiers.append(start)
        durations.append(duration)
        time_peak_intensities.append(time_peak_intensity)
    return calls_per_minute,tiers,durations,time_peak_intensities


dirname='./'
if 1>0:
    for file in os.listdir(dirname):
        if file.endswith(".TextGrid"):
            textgridfile=os.path.join(dirname, file)
        if file.endswith(".WAV"):
            soundfile=os.path.join(dirname, file)
            parselmouth_snd = parselmouth.Sound(soundfile)
            intensity = parselmouth_snd.to_intensity(minimum_pitch=minimumpitch,time_step=timestep)



tg = tgt.read_textgrid(textgridfile)
asealname='r17-292'
calls_per_minute,tiers,durations,time_peak_intensities=get_data_from_tier(tg,intensity,asealname,start_timepoint= 1,end_timepoint =1200000)
sublist=[]
old_onset=tiers[0]
for subint in tiers:
    if subint<=old_onset+4000:
         sublist.append(subint-old_onset)
    old_onset=subint
print len(sublist)
arraysublist=np.array(sublist)
arraytiers=np.array(tiers)
delta=np.diff(arraytiers)


print 'mean_IOIlo',np.mean(delta)
print 'std_IOIlo',np.std(delta)    
print len(delta)
print '==============='
#the 3900 msec threshold encompasses vocalizations up to 4 times the minimum IOI
filteredlist=[]
for val in delta:
    if val<3900:
        filteredlist.append(val)
arrayfilteredlist=np.array(filteredlist)
print 'mean_IOIsh',np.mean(arrayfilteredlist)
print 'std_IOIsh',np.std(arrayfilteredlist)
print len(arrayfilteredlist)
print '==============='

arraydurations=np.array(durations)
print 'mean_dur',np.mean(arraydurations)
print 'std_dur',np.std(arraydurations)
print len(arraydurations)
print '==============='

array_intens=np.array(time_peak_intensities)
IPIlo=np.diff(array_intens)
print 'mean_IPIlo',np.mean(IPIlo)
print 'std_IPIlo',np.std(IPIlo)
print len(IPIlo)
print '==============='

filteredIPIlist=[]
for value in IPIlo:
    if value<3900:
        filteredIPIlist.append(value)

IPIsh=np.array(filteredIPIlist)

print 'mean_IPIsh',np.mean(IPIsh)
print 'std_IPIsh',np.std(IPIsh)
print len(IPIsh)
print '==============='

np.savetxt('IOI_all.csv',delta, fmt="%s",delimiter=',')
np.savetxt('IOI_short.csv',arrayfilteredlist, fmt="%s",delimiter=',')
np.savetxt('durations_all.csv',arraydurations, fmt="%s",delimiter=',')
np.savetxt('IPI_all.csv',IPIlo, fmt="%s",delimiter=',')
np.savetxt('IPI_short.csv',IPIsh, fmt="%s",delimiter=',')


print 'comparing long distrib',ks_2samp(delta, IPIlo)
print 'comparing short distrib',ks_2samp(arrayfilteredlist, IPIsh)




print 'done'
